﻿using System;
using System.Linq;

namespace HeuristicLab.DataPreprocessing {

  public class StatisticInfo : IStatisticInfo {

    private IPreprocessingData preprocessingData;

    public StatisticInfo(IPreprocessingData thePreprocessingData) {
      preprocessingData = thePreprocessingData;
    }

    public int GetColumnCount() {
      return preprocessingData.Columns;
    }

    public int GetRowCount() {
      return preprocessingData.Rows;
    }

    public int GetNumericColumnCount() {
      int count = 0;
      foreach (var variableName in preprocessingData.VariableNames) {
        if (preprocessingData.IsType<double>(variableName)) {
          ++count;
        }
      }
      return count;
    }

    public int GetNominalColumnCount() {
      return preprocessingData.Columns - GetNumericColumnCount();
    }

    public int GetMissingValueCount() {
      int count = 0;
      foreach (var variableName in preprocessingData.VariableNames) {
        count += GetMissingValueCount(variableName);
      }
      return count;
    }

    public int GetMissingValueCount(string variableName) {
      if (preprocessingData.IsType<double>(variableName)) {
        return preprocessingData.GetValues<double>(variableName).Count(x => double.IsNaN(x));
      } else if (preprocessingData.IsType<string>(variableName)) {
        return preprocessingData.GetValues<string>(variableName).Count(x => string.IsNullOrEmpty(x));
      } else if (preprocessingData.IsType<DateTime>(variableName)) {
        return preprocessingData.GetValues<DateTime>(variableName).Count(x => x.Equals(DateTime.MinValue));
      } else {
        throw new ArgumentException("column with index: " + variableName + " contains a non supported type.");
      }
    }

    public T GetMin<T>(string variableName) where T : IComparable<T> {
      return preprocessingData.GetValues<T>(variableName).Min();
    }

    public T GetMax<T>(string variableName) where T : IComparable<T> {
      return preprocessingData.GetValues<T>(variableName).Max();
    }

    public double GetMedian(string variableName) {
      double median = double.NaN;
      if (preprocessingData.IsType<double>(variableName)) {
        median = preprocessingData.GetValues<double>(variableName).OrderBy(x => x).ElementAt(preprocessingData.Rows / 2);
      }
      return median;
    }

    public double GetAverage(string variableName) {
      double avg = double.NaN;
      if (preprocessingData.IsType<double>(variableName)) {
        avg = preprocessingData.GetValues<double>(variableName).Average();
      }
      return avg;
    }

    public T GetMostCommonValue<T>(string variableName) {
      return preprocessingData.GetValues<T>(variableName)
                              .GroupBy(x => x)
                              .OrderByDescending(g => g.Count())
                              .Select(g => g.Key)
                              .First();
    }


    public double GetStandardDeviation(string variableName) {
      double stdDev = double.NaN;
      if (preprocessingData.IsType<double>(variableName)) {
        double avg = GetAverage(variableName);
        stdDev = Math.Sqrt(preprocessingData.GetValues<double>(variableName).Sum(x => (x - avg) * (x - avg)) / (preprocessingData.Rows - 1));
      }
      return stdDev;
    }

    public int GetDifferentValuesCount<T>(string variableName) {
      return preprocessingData.GetValues<T>(variableName).GroupBy(x => x).Count();
    }
  }
}
